import httplib
import os
import subprocess
import sys
import time
import urllib
import urllib2
from authserv.test import *
from authserv.ratelimit import *
from authserv import protocol
from authserv import server

URL = '/api/1/auth'


def _relpath(x):
    return os.path.join(os.path.dirname(__file__), x)


class HTTPSClientAuthHandler(urllib2.HTTPSHandler):

    def __init__(self, cert, key):
        urllib2.HTTPSHandler.__init__(self)
        self.key = key
        self.cert = cert

    def https_open(self, req):
        # Rather than pass in a reference to a connection class, we pass in
        # a reference to a function which, for all intents and purposes,
        # will behave as a constructor
        return self.do_open(self.getConnection, req)

    def getConnection(self, host, timeout=300):
        return httplib.HTTPSConnection(host, key_file=self.key, cert_file=self.cert)


class SSLServerTest(unittest.TestCase):

    port = 63127

    ssl_ca = _relpath('testca/ca.pem')
    ssl_cert = _relpath('testca/certs/server.pem')
    ssl_key = _relpath('testca/private/server.key')
    client_cert = _relpath('testca/certs/client.pem')
    client_key = _relpath('testca/private/client.key')
    dhparams = _relpath('testca/dhparams')

    def setUp(self):
        self.users = {
            'user': FakeUser('user', 'pass'),
        }
        app = server.create_app(userdb=FakeUserDb(self.users),
                                mc=FakeMemcache(time.time))
        app.config.update({
                'TESTING': True,
                'DEBUG': True,
                })

        self._start_server()

        self.opener = urllib2.build_opener(
            HTTPSClientAuthHandler(self.client_cert, self.client_key))

    def tearDown(self):
        os.kill(self.pid, 15)

    def _start_server(self):
        """Run an SSL-enabled HTTP server as a separate process."""
        pid = os.fork()
        if pid == 0:
            print >>sys.stderr, 'starting server on port %d' % self.port
            server.run(None, '127.0.0.1', self.port, self.ssl_ca,
                       self.ssl_cert, self.ssl_key, self.dhparams)
        else:
            self.pid = pid
            time.sleep(0.2)

    def test_python_request_failure_without_cert(self):
        req = urllib2.Request('https://127.0.0.1:%d%s' % (self.port, URL),
                              data=urllib.urlencode(
                                  {'username': 'user',
                                   'password': 'pass',
                                   'service': 'svc',
                                   'source_ip': '127.0.0.1'}))
        self.assertRaises(urllib2.URLError, urllib2.urlopen, req)

    def test_python_auth_simple_ok(self):
        req = urllib2.Request('https://127.0.0.1:%d%s' % (self.port, URL),
                              data=urllib.urlencode(
                                  {'username': 'user',
                                   'password': 'pass',
                                   'service': 'svc',
                                   'source_ip': '127.0.0.1'}))
        resp = self.opener.open(req)
        data = resp.read()
        self.assertEquals(protocol.OK, data)

    def test_auth_client_suite(self):
        # Horrible hack: invoke the auth_client test suite.
        env = dict(os.environ)
        env['AUTH_SERVER'] = 'localhost:%d' % self.port
        retcode = subprocess.call(
            ['make', '-C', _relpath('../../pam'), 'check'],
            env=env)
        class _outputof(object):
            def __init__(self, p):
                self.path = p
            def __str__(self):
                with open(self.path) as fd:
                    return fd.read()
        self.assertEquals(retcode, 0, _outputof(_relpath('../../pam/test-suite.log')))
